# -*- coding: utf-8 -*-
"""
Created on Mon Aug 27 17:08:42 2018

@author: tb267
"""



class dataSet():
    def __init__(self,timeData=None,freqData=None,tfData=None,sonoData=None,settings=None,metaData=None):
        self.timeData = timeData
        self.freqData = freqData
        self.tfData   = tfData
        self.sonoData = sonoData
        self.settings = settings
        self.metaData = metaData
        
    def __repr__(self):
        text = '<dataSet class containing: '
        if repr(self.timeData) != 'None':
            text += repr(self.timeData)
        if repr(self.freqData) != 'None':
            text += repr(self.freqData)
        if repr(self.tfData) != 'None':
            text += repr(self.tfData)
        if repr(self.sonoData) != 'None':
            text += repr(self.sonoData)
        if repr(self.metaData) != 'None':
            text += repr(self.metaData)
        
        text += '>'
        return text
        
        
class timeData():
    def __init__(self,time_axis,time_data,settings):
        self.time_axis = time_axis
        self.time_data = time_data  
        self.settings = settings
        
    def __repr__(self):
        return "<timeData>"

        
class freqData():
    def __init__(self,freq_axis,freq_data,settings):
        self.freq_axis = freq_axis
        self.freq_data = freq_data
        self.settings = settings
        
    def __repr__(self):
        return "<freqData>"
        
class tfData():
    def __init__(self,freq_axis,tf_data,settings):
        self.freq_axis = freq_axis
        self.tf_data = tf_data
        self.settings = settings
        
    def __repr__(self):
        return "<tfData>"
        
class sonoData():
    def __init__(self,time_axis,freq_axis,sono_data,settings):
        self.time_axis = time_axis
        self.freq_axis = freq_axis
        self.sono_data = sono_data
        self.settings = settings
        
    def __repr__(self):
        return "<sonoData>"
        
class metaData():
    def __init__(self, timestamp=None, timestring=None, units=None, channel_cal_factors=None, tf_cal_factors = None):
        self.timestamp = timestamp
        self.timestring = timestring
        self.units = units
        self.channel_cal_factors = None
        self.tf_cal_factors = None
        
    def __repr__(self):
        return "<metaData>"


# -*- coding: utf-8 -*-
"""
Created on Mon Aug 27 13:28:39 2018

@author: tb267
"""

import pyaudio
import numpy as np
import pyqtgraph as pg






class mySettings(object):
    '''
    A class that stores the acquisition settings.
    
    Attributes:
    --------------       
    channels: int
        Number of Channels
    fs: int
        Sampling Frequency
    nbits: int
        Number of bits - either 8, 16, 24 or 32
    chunk_size: int
        Number of samples obtained from each channel in one chunk
    num_chunks: int
        Number of chunks to store in circular buffer
    view_time: float
        Overrides num_chunks to display view_time in seconds for oscilloscope
    stored_time: float
        Length of the pre-trigger when the space button is hit, in seconds
    device_index: int
        device index, will prompt if not specified
    init_view_time: bool
        flag for time domain view in oscilloscope
    init_view_freq: bool
        flag for frequency domain view in oscilloscope
    init_view_levels: bool
        flag for channel levels view in oscilloscope
    '''
        
    def __init__(self, *, 
                 channels=2, 
                 fs=44100, 
                 nbits=16, 
                 chunk_size=1024, 
                 num_chunks=6,
                 view_time=None,
                 stored_time=2,
                 device_index=None,
                 init_view_time=True,
                 init_view_freq=True,
                 init_view_levels=True):
        
        self.channels=channels
        self.fs=fs
        self.chunk_size=chunk_size
        self.num_chunks=num_chunks
        self.view_time=view_time
        self.nbits=nbits
        self.stored_time=stored_time
        self.device_index=device_index
        self.init_view_time=init_view_time
        self.init_view_freq=init_view_freq
        self.init_view_levels=init_view_levels
        
        ### derived settings
        if view_time != None:
            self.num_chunks = int(np.ceil(view_time*fs/chunk_size))
            
        self.format = eval('pyaudio.paInt'+str(self.nbits))
        self.device_name = None # until initialise stream
        
    
    def __repr__(self):
        return '<mySettings class>'
        
    
        
        
def set_plot_colours(channels):
    '''
    Returns a list of RGB colours depending on the number of channels required.
    '''
    #TODO: Accessible colours
    val = [0.0,0.5,1.0]
    colour = np.array([[255,0,0,255],[0,255,0,255],[0,0,255,255]], dtype = np.ubyte)
    plot_colourmap =  pg.ColorMap(val,colour)
    c_list = plot_colourmap.getLookupTable(nPts =channels,alpha=True)
    return c_list 



# -*- coding: utf-8 -*-
"""
Created on Mon Aug 27 14:32:35 2018

@author: tb267
"""
import sys

from . import settings
from . import plotting
from . import logdata

import numpy as np
import tkinter as tk
from tkinter import filedialog


def read_data():
    '''
    Gives the user a filedialog to select a .npy file.
    '''
    
    ##fromfile means extracted 
    open_root = tk.Tk()
    open_root.attributes('-topmost', 1)
    #open_root.attributes('-topmost', 0) # commented to force user to save or cancel
    open_root.filename =  filedialog.askopenfilename()
    if open_root.filename: # askopenfilename returns `False` if dialog closed with "cancel".
        d=np.load(open_root.filename)
        dataset = d[0]
        open_root.destroy()
    else:
        open_root.destroy()
        return

    return dataset
#    
#    fromfile_settings=fromfile_data_array[0]
#    fromfile_data=fromfile_data_array[1]
#    fs=fromfile_settings.fs
#    n_samp=len(fromfile_data[:,0])
#    dt=1/fs
#    t_samp=n_samp*dt
#    fromfile_t_axis= np.arange(0,t_samp,dt)
#    fromfile_freq_axis=np.fft.rfftfreq(len(fromfile_t_axis),1/fs)
#    fromfile_time_data_windowed=np.zeros_like(fromfile_data)
#    fromfile_num_chunks=int(np.ceil((fromfile_settings.stored_time*fromfile_settings.fs)/fromfile_settings.chunk_size))
#    if (fromfile_num_chunks*fromfile_settings.chunk_size)%2==0:
#        fromfile_freq_data=np.zeros(shape=(int(((fromfile_num_chunks*fromfile_settings.chunk_size+2)/2)+1),fromfile_settings.channels))
#    else:
#        fromfile_freq_data=np.zeros(shape=(int((fromfile_num_chunks*fromfile_settings.chunk_size+2+1)/2),fromfile_settings.channels))
#    for i in range(fromfile_settings.channels) :  
#        fromfile_time_data_windowed[:,i] = fromfile_data[:,i] * np.hanning(np.shape(fromfile_data)[0])
#        fromfile_freq_data[:,i] = 20 * np.log10(np.abs(np.fft.rfft(fromfile_time_data_windowed[:,i]))/len(fromfile_time_data_windowed[:,i]))
#        
#    
#    fromfile_time_line = figure(title="Audio Signal Vs. Time", x_axis_label='Time (s)', y_axis_label='Normalised Amplitude')
#    fromfile_freq_line = figure(title="Power Vs Frequency Domain", x_axis_label='Frequency (Hz)', y_axis_label='Power Spectrum (dB)')
#
#    fromfile_time_line.add_tools(pltm.HoverTool())
#    fromfile_freq_line.add_tools(pltm.HoverTool())
#   
#    #this holds on to a label when tapped on but doesn't seem necessary for now
#    #fromfile_time_line.add_tools(pltm.TapTool())
#    #fromfile_freq_line.add_tools(pltm.TapTool())
#    
#    for i in range(fromfile_settings.channels):
#        #plot the time and fft domains
#        fromfile_time_line.line(fromfile_t_axis, fromfile_data[:,i], legend='Channel'+str(i),line_color=tuple(setup.set_plot_colours(fromfile_settings.channels)[i,:]), line_width=0.7)
#        fromfile_freq_line.line(fromfile_freq_axis, fromfile_freq_data[:,i], legend='Channel'+str(i),line_color=tuple(setup.set_plot_colours(fromfile_settings.channels)[i,:]), line_width=0.7)
#    
#    #have to determine the legend.click_policy after defining the line
#    fromfile_time_line.legend.click_policy="hide"
#    fromfile_freq_line.legend.click_policy="hide"
#    
#    #resets the output of the implicit bokeh current document.
#    #Initially, when read_data() was called severally, new figures would be added to the older ones.
#    reset_output()
#    
#    show(row(fromfile_time_line,fromfile_freq_line))




#TODO: Supressing little Tk window. Note that it already gets destroyed after performing its function
# to supress little tk window
#root=tk.Tk()
#root.withdraw()
            
#currently stays as the topmost dialog until exited.
#below line should fix it but instead hides it back
#try adding a pause inbetween the two lines
#tk_window.attributes('-topmost', 0)
#TODO:Sent dialog to the top but without being permanently there.

 
    
            



from .settings import mySettings
from .file import read_data
from .oscilloscope import oscilloscope, recorder
from .logdata import dataSet, timeData, freqData, tfData, sonoData, metaData
from .plotting import plotdata
from .analysis import convert_to_frequency





# -*- coding: utf-8 -*-
"""
Created on Tue Aug 28 19:04:14 2018

@author: tb267
"""



from . import settings
from . import file
from . import logdata


###----------------------------------------------------------------------------
import numpy as np
import scipy as sp

###----------------------------------------------------------------------------
import matplotlib
import matplotlib.pyplot as plt
matplotlib.rcParams.update({'font.size': 12,'font.family':'serif'})
###----------------------------------------------------------------------------


class plotdata(object):
    def __init__(self,data):
        '''
        Args:
            data: plots data which can be class of type:
                    logdata.dataSet
                    logdata.timeData
                    logdata.freqData
        '''
        
        
        if data.__class__.__name__ == 'dataSet':
            tdata = data.timeData
            fdata = data.freqData
            self.dataset = data
            
        elif data.__class__.__name__  == 'timeData':
            tdata = data
            fdata = None
            self.dataset = logdata.dataSet(timeData=tdata,settings=tdata.settings)
            
        elif data.__class__.__name__  == 'freqData':
            tdata = None
            fdata = data
            self.dataset = logdata.dataSet(freqData=fdata,settings=fdata.settings)
            
        if tdata != None and fdata != None:
            ### plot time and frequency domain data together
            self.fig, self.ax = plt.subplots(2, 1,figsize = (9,5),dpi=100)
            
            self.ax[0].set_xlabel('Time (s)')
            self.ax[0].set_ylabel('Normalised Amplitude')
#            self.ax[0].set_xlim(fdata.settings.time_range)
            self.ax[0].axvspan(fdata.settings.time_range[0], fdata.settings.time_range[1], color='blue', alpha=0.25)
            self.ax[0].grid()
            self.ax[1].set_xlabel('Frequency (Hz)')
            self.ax[1].set_ylabel('Amplitude (dB)')
            self.ax[1].grid()
            
            
            for n in range(data.settings.channels):
                self.ax[0].plot(tdata.time_axis,tdata.time_data[:,n],'-',linewidth=1,color = settings.set_plot_colours(data.settings.channels)[n,:]/255,label='ch '+str(n))
                self.ax[1].plot(fdata.freq_axis,20*np.log10(np.abs(fdata.freq_data[:,n])),'-',linewidth=1,color = settings.set_plot_colours(data.settings.channels)[n,:]/255,label='ch '+str(n))
                
            self.ax[0].legend()
            self.ax[1].legend()
            
        elif tdata != None:
            ### plot time domain data
            self.fig, self.ax = plt.subplots(figsize = (9,5),dpi=100)
        
            self.ax.set_xlabel('Time (s)')
            self.ax.set_ylabel('Normalised Amplitude')
            self.ax.grid()
            
            for n in range(data.settings.channels):
                self.ax.plot(tdata.time_axis,tdata.time_data[:,n],'-',linewidth=1,color = settings.set_plot_colours(data.settings.channels)[n,:]/255,label='ch '+str(n))
                
            self.ax.legend()
            
        elif fdata != None:
            ### plot frequency domain data
            self.fig, self.ax = plt.subplots(figsize = (9,5),dpi=100)
        
            self.ax.set_xlabel('Frequency (Hz)')
            self.ax.set_ylabel('Amplitude (dB)')
            self.ax.grid()
            
            for n in range(data.settings.channels):
                self.ax.plot(fdata.freq_axis,20*np.log10(np.abs(fdata.freq_data[:,n])),'-',linewidth=1,color = settings.set_plot_colours(data.settings.channels)[n,:]/255,label='ch '+str(n))
                
            self.ax.legend()
            

        
        plt.show()
        
        
        
#class plot_timeData
    
    
    





# -*- coding: utf-8 -*-
"""
Created on Fri Aug  3 11:27:29 2018

@authors: ae407, tb267
"""      
import sys

from . import settings
from . import file
from . import logdata
from . import plotting

import pyaudio
import numpy as np
import pyqtgraph as pg
import tkinter as tk
from pyqtgraph.Qt import QtGui, QtCore
import matplotlib
import matplotlib.pyplot as plt
from tkinter import filedialog
import time
import datetime


class oscilloscope(object):
    
    app = None
    
    def __init__(self,settings):
        '''Creates an Oscilloscope
        Args:
            settings: An object of the class mySettings
        '''
        self.settings = settings
            
        rec = recorder(settings)
        audio_stream,audio = rec.init_pyaudio(settings)
        self.rec = rec
        self.audio_stream = audio_stream
        self.audio = audio    
        
        if (sys.flags.interactive != 1) or not hasattr(QtCore, 'PYQT_VERSION'):
            
            
            self.create_figure()
            
            self.win.sigKeyPress.connect(self.keyPressed)
    
            self.timer = QtCore.QTimer()
            self.timer.timeout.connect(lambda:self.update(audio_stream)) # update figure and buffer
            self.timer.start(0)
            oscilloscope.app.instance().exec_()
            
            
    def create_figure(self):
        '''
        Creates a figure which is an object of the class KeyPressWindow.
        
        '''
        pg.setConfigOption('background', 'w')
        if oscilloscope.app == None:
            oscilloscope.app = QtGui.QApplication([])
            
        self.win = KeyPressWindow(self)
#        self.win.setWindowIcon(QtGui.QIcon('icon.png'))
        
        
        window_geometry = self.win.geometry()
        
#        screen_width = tk.Tk().winfo_screenwidth()
#        screen_height = tk.Tk().winfo_screenheight()
        
        self.win.setGeometry(100,100,800,600)
        
        self.win.showMinimized()
        self.win.showNormal()


        self.win.setWindowTitle("Oscilloscope (to save to new filename press 's', to autosave press 'space')")
        self.view_time = self.settings.init_view_time
        self.view_freq = self.settings.init_view_freq
        self.view_levels = self.settings.init_view_levels
        
        self.toggle_view()
        
        self.data_saved_counter = 0 #  to indicate not yet saved file
           
            

    def toggle_view(self):
        '''
        Switches between views, triggered by keypress
        '''
        self.win.clear()
        
        
        if self.view_time == True:
            self.time_plot()
            
        if self.view_freq == True:
            self.freq_plot()
            
        if self.view_levels == True:
            self.levels_plot()
        
        
            
            
    
        
    
    def time_plot(self):
        # create a plot for the time domain
        self.view_time = True
        self.win.nextRow()
        self.osc_time_line = self.win.addPlot(title="Time Domain (toggle with 'T')")
        if self.settings.channels==1:
            self.osc_time_line.enableAutoRange()
        else:
            self.osc_time_line.setYRange(-1,self.settings.channels)
        self.osc_time_line.setXRange(self.rec.osc_time_axis[0],self.rec.osc_time_axis[-1])
        self.osc_time_line.showGrid(True, True)
        self.osc_time_line.addLegend()
        self.osc_time_line.setLabel('left','Normalised Amplitude')
        self.osc_time_line.setLabel('bottom','Time (s)')  
        
        ax=self.osc_time_line.getAxis('left')
        ax.setTickSpacing(1,1)
        
        self.osc_time_lineset={}
        for i in range(self.settings.channels):
            pen_ = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:])
            self.osc_time_lineset[i]=self.osc_time_line.plot(pen=pen_, name='Channel '+str(i))
        
#        self.win.FillBetweenItem(curve1=osc_time_lineset[0], curve2=osc_time_lineset[1])
        
    def freq_plot(self):
        # create a plot for the frequency domain
        self.view_freq = True
        self.win.nextRow()
        self.osc_freq_line = self.win.addPlot(title="Frequency Domain (toggle with 'F')") 
        self.osc_freq_line.enableAutoRange()
        self.osc_freq_line.setXRange(self.rec.osc_freq_axis[0],self.rec.osc_freq_axis[-1])
        self.osc_freq_line.showGrid(True, True)
        self.osc_freq_line.addLegend()
        self.osc_freq_line.setLabel('left','Power Spectrum (dB)')
        self.osc_freq_line.setLabel('bottom','Frequency (Hz)')
        
        self.osc_freq_lineset={}
        for i in range(self.settings.channels):
            pen_ = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:])
            self.osc_freq_lineset[i]=self.osc_freq_line.plot(pen=pen_, name='Channel'+str(i))
            
    def levels_plot(self):
        # create a plot for the frequency domain
        self.view_levels = True
        self.win.nextRow()
        
        self.osc_levels_line = self.win.addPlot(title="Channel Levels (toggle with 'L')")
        self.osc_levels_line.setYRange(0,1)
        self.osc_levels_line.setXRange(-0.5,self.settings.channels-0.5)
        self.osc_levels_line.showGrid(False,True)
        self.osc_levels_line.setLabel('left','Normalised Amplitude')
        self.osc_levels_line.setLabel('bottom','Channel Index')
        
        
        ax=self.osc_levels_line.getAxis('bottom')
        ax.setTickSpacing(1,1)    
#        ax.showLabel(show=True)
#        self.osc_levels_line.setTicks(np.arange(self.settings.channels))
        self.osc_levels_lineset={}
        for i in range(self.settings.channels):
            pen_ = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:],width=3)
            pen_peak = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:],width=3)
            self.osc_levels_lineset[i]=self.osc_levels_line.plot(pen=pen_, name='vertical')
            self.osc_levels_lineset[self.settings.channels+i]=self.osc_levels_line.plot(pen=pen_, name='top')
            self.osc_levels_lineset[2*self.settings.channels+i]=self.osc_levels_line.plot(pen=pen_peak, name='peak hold')    
#            self.osc_levels_lineset[3]=self.osc_levels_line.plot(pen=pen_, name='Channel')
        
        self.osc_levels_peak_hold = np.zeros(self.settings.channels)
        self.time_last_changed = np.zeros(self.settings.channels)
        
    
    def update(self,audio_stream):
        '''
        Updates plots with incoming data from __call__. 
        Called with a 0s interval by QTimer.
        
        '''
        if self.view_levels == True:
            self.osc_levels_rms = np.sqrt(np.mean(self.rec.osc_time_data**2,axis=0))
            self.osc_levels_max = np.max(np.abs(self.rec.osc_time_data),axis=0)
            changed_indices = self.osc_levels_peak_hold < self.osc_levels_max
            self.time_last_changed[changed_indices] = time.time()
            self.osc_levels_peak_hold = np.maximum(self.osc_levels_peak_hold,self.osc_levels_max)
            self.osc_levels_peak_hold[time.time()-self.time_last_changed>2] = 0
            
                
        for i in range(self.settings.channels):
            offset=i
            if self.view_time == True:
                self.osc_time_lineset[i].setData(self.rec.osc_time_axis,self.rec.osc_time_data[:,i] + offset)
            
            if self.view_freq == True:
                # calculate the FFT
                self.rec.osc_time_data_windowed[:,i] = self.rec.osc_time_data[:,i] * np.blackman(np.shape(self.rec.osc_time_data)[0])
                self.rec.osc_freq_data[:,i] = 20 * np.log10(np.abs(np.fft.rfft(self.rec.osc_time_data_windowed[:,i]))/len(self.rec.osc_time_data_windowed[:,i]))
                self.osc_freq_lineset[i].setData(self.rec.osc_freq_axis,self.rec.osc_freq_data[:,i])
                
            if self.view_levels == True:
                self.osc_levels_lineset[i].setData([i,i],[0,self.osc_levels_max[i]])
                self.osc_levels_lineset[self.settings.channels+i].setData([i-0.3,i+0.3],self.osc_levels_max[i]*np.ones(2))
                
                if self.osc_levels_peak_hold[i] > 0.98:
                    pen_peak = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:],width=10)
                else:
                    pen_peak = pg.mkPen(color=settings.set_plot_colours(self.settings.channels)[i,:],width=3)
#                self.osc_levels_lineset[2*self.settings.channels+i]=self.osc_levels_line.plot(pen=pen_peak, name='peak hold')
                self.osc_levels_lineset[2*self.settings.channels+i].setData([i-0.3,i+0.3],self.osc_levels_peak_hold[i]*np.ones(2),pen=pen_peak)
#                self.osc_levels_lineset[3].setData(np.arange(2),np.ones(2))
                
                
            #updates for the stored
            self.rec.stored_time_data_windowed[:,i] = self.rec.stored_time_data[:,i] * np.blackman(np.shape(self.rec.stored_time_data)[0])
            self.rec.stored_freq_data[:,i] = 20 * np.log10(np.abs(np.fft.rfft(self.rec.stored_time_data_windowed[:,i]))/len(self.rec.stored_time_data_windowed[:,i]))
    

        
            
            
    #KeyPressed function within osciolloscpe since can only take one argument        
    def keyPressed(self, evt):
        '''
        Upon a Space Bar press, makes a copy of data from the past stored_time seconds,plots it in Bokeh and gives the user an option to save it.
        '''
        
        if evt.key() == QtCore.Qt.Key_T:

            if self.view_freq != False or self.view_levels != False:
                print('toggled time domain view')
                self.view_time = not self.view_time
                self.toggle_view()
            else:
                print('toggling all views off is prevented')
            
            
        if evt.key() == QtCore.Qt.Key_F:

            if self.view_time != False or self.view_levels != False:
                print('toggled frequency domain view')
                self.view_freq = not self.view_freq
                self.toggle_view()
            else:
                print('toggling all views off is prevented')
            
            
        if evt.key() == QtCore.Qt.Key_L:
            if self.view_time != False or self.view_freq != False:
                print('toggled levels view')
                self.view_levels = not self.view_levels
                self.toggle_view()
            else:
                print('toggling all views off is prevented')
        
        
        if evt.key() == QtCore.Qt.Key_Space or evt.key() == QtCore.Qt.Key_S:
            
            stored_time_data_copy=np.copy(self.rec.stored_time_data)
            t = datetime.datetime.now()
            timestring = '_'+str(t.year)+'_'+str(t.month)+'_'+str(t.day)+'_at_'+str(t.hour)+'_'+str(t.minute)+'_'+str(t.second)
            
            
            print("key press trigger: saving data to file in working directory")
            
            ### make into dataset
            
            fs=self.settings.fs
            n_samp=len(stored_time_data_copy[:,0])
            dt=1/fs
            t_samp=n_samp*dt
            t_axis= np.arange(0,t_samp,dt)
#            f_axis=np.fft.rfftfreq(len(t_axis),1/fs)
#            freq_data=np.fft.rfft(stored_time_data_copy,axis=0)
            
            
            t_data = logdata.timeData(t_axis,stored_time_data_copy,self.settings)
            metadata = logdata.metaData(timestamp=t,timestring=timestring)
            dataset = logdata.dataSet(timeData=t_data, settings=self.settings, metaData=metadata)
            
            plotting.plotdata(dataset)
            
            data_array=np.array([dataset])
            
            if evt.key() == QtCore.Qt.Key_S:
                self.data_saved_counter = 0
            
            if self.data_saved_counter == 0:

                self.save_root = tk.Tk()
                self.save_root.attributes('-topmost', 1)
                self.save_root.filename =  filedialog.asksaveasfilename(defaultextension=".npy", filetypes=(("npy file", "*.npy"),("All Files", "*.*") ))
                
                if self.save_root.filename is '': # asksaveasfile returns `None` if dialog closed with "cancel".
                    self.save_root.destroy()
                else:
                    filename = self.save_root.filename.replace(".npy","")
                    np.save(filename+".npy",data_array)
                    self.save_root.destroy()
                    
                    print("... file saved as " + filename + ".npy")
                    self.data_saved_counter += 1

            else:
                filename = self.save_root.filename.replace(".npy","")
                np.save(filename+'_'+str(self.data_saved_counter)+".npy",data_array)
                self.data_saved_counter += 1
                print("... file saved as " + filename+'_'+str(self.data_saved_counter)+".npy")

                
            
        
        
class recorder(object):
    def __init__(self,settings):
        self.settings = settings
        self.osc_time_axis=np.arange(0,(self.settings.num_chunks*self.settings.chunk_size)/self.settings.fs,1/self.settings.fs)
        self.osc_freq_axis=np.fft.rfftfreq(len(self.osc_time_axis),1/self.settings.fs)
        self.osc_time_data=np.zeros(shape=((self.settings.num_chunks*self.settings.chunk_size),self.settings.channels))  
        self.osc_time_data_windowed=np.zeros_like(self.osc_time_data)
        if (self.settings.num_chunks*self.settings.chunk_size)%2==0:
            self.osc_freq_data=np.zeros(shape=(int(((self.settings.num_chunks*self.settings.chunk_size)/2)+1),self.settings.channels))
        else:
            self.osc_freq_data=np.zeros(shape=(int((self.settings.num_chunks*self.settings.chunk_size+1)/2),self.settings.channels))
        
        #rounds up the number of chunks needed in the pretrig array    
        self.stored_num_chunks=int(np.ceil((self.settings.stored_time*self.settings.fs)/self.settings.chunk_size))
        #the +2 is to allow for the updating process on either side
        self.stored_time_data=np.zeros(shape=((self.stored_num_chunks*self.settings.chunk_size)+2,self.settings.channels))
        self.stored_time_data_windowed=np.zeros_like(self.stored_time_data)
        #note the +2s to match up the length of stored_num_chunks
        #formula used from the np.fft.rfft documentation
        if (self.stored_num_chunks*self.settings.chunk_size)%2==0:
            self.stored_freq_data=np.zeros(shape=(int(((
                    self.stored_num_chunks*self.settings.chunk_size+2)/2)+1),self.settings.channels))
        else:
            self.stored_freq_data=np.zeros(shape=(int((
                    self.stored_num_chunks*self.settings.chunk_size+2+1)/2),self.settings.channels))
            
            
    def __call__(self,in_data, frame_count, time_info, status):
        '''
        Obtains data from the audio stream.
        '''
        self.osc_data_chunk = (np.frombuffer(in_data, dtype=eval('np.int'+str(self.settings.nbits)))/2**(self.settings.nbits-1))
        self.osc_data_chunk=np.reshape(self.osc_data_chunk,[self.settings.chunk_size,self.settings.channels])
        for i in range(self.settings.channels):
            self.osc_time_data[:-(self.settings.chunk_size),i] = self.osc_time_data[self.settings.chunk_size:,i]
            self.osc_time_data[-(self.settings.chunk_size):,i] = self.osc_data_chunk[:,i]
            self.stored_time_data[:-(self.settings.chunk_size),i] = self.stored_time_data[self.settings.chunk_size:,i]
            self.stored_time_data[-(self.settings.chunk_size):,i] = self.osc_data_chunk[:,i]
        return (in_data, pyaudio.paContinue)
    
    
    def init_pyaudio(self,settings,_input_=True,_output_=False):
        '''
        Initialises an audio stream. Gives the user a choice of which device to access.
        '''
        global audio,audio_stream
        
        audio = pyaudio.PyAudio()
        
        if settings.device_index == None:
    
            device_count = audio.get_device_count()
            print ('Number of devices available is: %i' %device_count)
            print ('')
            print('Devices available, by index:')
            print ('')
            for i in range(device_count):
                device = audio.get_device_info_by_index(i)
                print(device['index'], device['name'])
            print ('')
            default_device = audio.get_default_input_device_info()
            print('Default device is: %i %s'
                  %(default_device['index'],default_device['name']))
            print ('')
            settings.device_index=int(input('Insert index of required device:'))
            
        settings.device_name = audio.get_device_info_by_index(settings.device_index)['name']
        settings.device_full_info = audio.get_device_info_by_index(settings.device_index)
        
        print(settings.device_full_info['index'], settings.device_full_info['name'])
        print("Selected device: %i : %s" %(settings.device_index,settings.device_name))    
        
        audio_stream = audio.open(format=settings.format,
                                        channels=settings.channels,
                                        rate=settings.fs,
                                        input=_input_,
                                        output=_output_,
                                        frames_per_buffer=settings.chunk_size,
                                        input_device_index=settings.device_index,
                                        stream_callback=self.__call__)  
        audio_stream.start_stream()
        
        return (audio_stream,audio)
        
    
    def end_pyaudio(self,audio_stream,audio):
        '''
        Closes an audio stream.
        '''
        if not audio_stream.is_stopped():
            audio_stream.stop_stream()
        else: 
            pass
        audio_stream.close()
        audio.terminate()
        
        

    

     

    
class KeyPressWindow(pg.GraphicsWindow):
    '''
    A subclass of pyQtGraph GraphicsWindow that emits a signal when a key is pressed.
    
    '''
    sigKeyPress = QtCore.pyqtSignal(object)
    

    def __init__(self, oscilloscope, *args, **kwargs):
        '''
        Re-implmented from parent.
        '''
        super().__init__(*args, **kwargs)
        self.oscilloscope = oscilloscope

    def keyPressEvent(self, evt):
        '''
        Emits a signal upon a key press 
        '''
        self.scene().keyPressEvent(evt)
        self.sigKeyPress.emit(evt)
        
    def closeEvent(self,event):
        '''
        Stops QTimer,exits QApplication and closes the audio stream when the user exits the oscilloscope window.
        '''
        
        self.oscilloscope.timer.stop()
        #self.oscilloscope.app.exit()
        self.close()
        self.oscilloscope.rec.end_pyaudio(self.oscilloscope.audio_stream,self.oscilloscope.audio)
        


   

         


# -*- coding: utf-8 -*-
"""
Created on Mon Aug 13 15:37:24 2018

@author: ae407
"""
import numpy as np
import scipy as sp
import peakutils
import pydvma as dvma
        

DEFAULT_SETTINGS = dvma.mySettings(channels=1,fs=8000,stored_time=2,view_time=2,device_index=1)


class beam(object):
    ''' Defines beam data for m1 lab'''
    
    def __init__(self,*,L,b,d,M):
        """
        Args:
            L: length (m)
            b: width (m)
            d: thickness (m)
            M: mass (kg)
            
        Note units are in m or kg
            
        """
        self.L = L
        self.b = b
        self.d = d
        self.M = M
        self.rho = self.M/(self.L*self.b*self.d)
        self.I = (self.b*(self.d**3))/12
        self.A = self.b*self.d
        self.alphaL = np.array([4.730, 7.853, 10.996, 14.137])
        

def E_calculator(*,beam,frequency,mode):
    '''
    Args:
        beam (class): Beam data object
        frequency: frequency of mode in Hz.
        mode (int): mode number

    Returns:
        E: Calculation of Young's Modulus
    '''
    
    if type(mode)!=int:
        print('Insert an integer for mode')
        return
    elif mode < 1 or mode > 4:
        print('Invalid mode number. Please give a number between 1 and 4')
        return
    
    alpha = beam.alphaL[mode-1] / beam.L
    
    E = ((2*np.pi*frequency)**2)*((beam.L/alpha)**4)*beam.rho*beam.A/beam.I
    print('The Young\'s Modulus calculated from Mode %i is %.2f GPa' %(mode,E*1e-9))  
    print('Density= %.2f kgm^-3' % beam.rho)
    return E
        
    
    
#####some data####
"""
beam_data=m1.beam_data(200,25,1.37,16)
f1=153.45
f2=424.22
f3=833.09
f4=1387.50
"""


def find_peaks(freq_data,channel=0,threshold=0,freq_range=None):
    '''
    Returns a list of the n most prominent peaks in plot.
    
    Args:
        freq_data:
        freq_range:
    '''
    if freq_range == None:
        ### use all data
        freq_range = freq_data.freq_axis[[0,-1]]
        
    elif freq_range.__class__.__name__ == 'plotdata':
        threshold=freq_range.ax.get_ybound()
        threshold=threshold[0]
        freq_range=freq_range.ax.get_xbound()
        
        

    s1 = freq_data.freq_axis >= freq_range[0]
    s2 = freq_data.freq_axis <= freq_range[1]
    selection = s1 & s2
    
    faxis = freq_data.freq_axis[selection]
    fdata = freq_data.freq_data[selection,:]
    fdata = fdata[:,channel]
    
    peaks = sp.signal.find_peaks(np.abs(fdata))
    peaks = peaks[0]
    
    df = freq_data.freq_axis[1]
    peaks = peakutils.indexes(20*np.log10(np.abs(fdata)), thres=threshold, min_dist=np.round(10/df),thres_abs=True)

    peak_frequencies = faxis[peaks]
    peak_amplitudes= np.abs(fdata[peaks])
    
#    keep = 20*np.log10(np.abs(peak_amplitudes)) > threshold
#    
##    largest = np.argsort(-peak_amplitudes)
##    largest = largest[0:n]
##    peak_frequencies = peak_frequencies[largest]
##    peak_amplitudes = peak_amplitudes[largest]
##    indices = np.argsort(peak_frequencies)
#    peak_frequencies = peak_frequencies[keep]
#    peak_amplitudes  = peak_amplitudes[keep]
    
    if len(peak_frequencies) < 10:    
        print('The most significant peaks above %s dB in the specified range are at the following frequencies:' % str(np.round(threshold,decimals=1)))
        print(round2nsf(peak_frequencies,4))
    else:
        print('%i peaks have been found. The first 10 are:' % len(peak_frequencies))
        print(round2nsf(peak_frequencies[0:10],4))
        print('To reduce the number of peaks, use a higher threshold. If using a plot to specifiy the threshold then use the zoom to view only the peaks and not the noisy data underneath.')
        
    return peak_frequencies, peak_amplitudes


def round2nsf(x,nsf):
    '''
    round x to nsf significant figures
    '''
    
    temp = 10**(np.floor(np.log10(np.abs(x)) - nsf + 1));
    y = np.round(x/temp)*temp;
    
    return y
